# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template1LevFact(Template):
    'Fits a 1-level factor model, allowing the same fixed effects across responses, and constraints on either the factor loadings or variances. IMPORTANT: if using the e-STAT engine, you need to opt for "No optimisation steps" (see Settings via the main page).'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal', 'Factor analysis' ]
    engines = ['eStat', 'MLwiN_MCMC']

    inputs = '''
y = DataMatrix('Responses: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
NumFact = Integer('Number of factors: ')

for j in range(0, int(NumFact)):
    context['fact' + str(j+1)] = ParamVector(monitor=False)
    for i in range(0, len(y)):
        context['constr_load' + str(j+1) + '_' + str(i+1)] = Boolean('Is loading ' + str(i+1) + ' for factor ' + str(j+1) + ' constrained? ')
        if not context['constr_load' + str(j+1) + '_' + str(i+1)]:
            context['load' + str(j+1) + '_' + str(i+1)] = ParamScalar()
        else:
            context['load' + str(j+1) + '_' + str(i+1)] = Text('value:')

    context['constr_fact' + str(j+1)] = Boolean('Is factor ' + str(j+1) + ' variance constrained? ')
    if not context['constr_fact' + str(j+1)]:
        context['tau_fact' + str(j+1)] = ParamScalar()
        context['sigma2_fact' + str(j+1)] = ParamScalar(modelled = False)
    else:
        context['tau_fact' + str(j+1)] = Text('value: ')

for i in range(0, len(y)):  
    context['tau_u2_' + str(i+1)] = ParamScalar()
    context['sigma2_u2_' + str(i+1)] = ParamScalar(modelled = False)
    context['miss'+y[i]] = ParamVector(monitor=False)
    context['beta' + '_' + str(i+1)] = ParamVector(parents=[x], as_scalar=True)
deviance = ParamScalar(modelled=False, customstep=True)
'''      

    model = '''
<% numfacts = int(NumFact) %>
model {
    for (i in 1:length(${y[0]})) {
        % for i in range(0, len(y)):
        miss${y[i]}[i] ~ dnorm(mu${i+1}[i], tau_u2_${i+1})
        mu${i+1}[i] <- ${mmult(x, 'beta' + '_' + str(i+1), 'i')} \\
        % for j in range(0, numfacts):
        + load${j+1}_${i+1} * fact${j+1}[i] 
        % endfor
        dummy${y[i]}[i] ~ ddummy(miss${y[i]}[i])
        % endfor
    }
    % for i in range(0, len(y)):
    % for j in range(0, numfacts):
    % if not context['constr_load' + str(j+1) + '_' + str(i+1)]:
    load${j+1}_${i+1} ~ dflat()
    % else:
    load${j+1}_${i+1} <- ${context['load' + str(j+1) + '_' + str(i+1)]}
    % endif
    % endfor
    % endfor
    
    for (i in 1:length(${y[0]})) {
        % for j in range(0, numfacts):
        fact${j+1}[i] ~ dnorm(0, tau_fact${j+1})
        % endfor
    }
    
    # Priors
    % for i in range(0, len(y)):
    % for j in range(0, len(x)):
    beta_${i+1}_${j} ~ dflat()
    % endfor
    % endfor
    % for i in range(0, len(y)):
    tau_u2_${i+1} ~ dgamma(0.001000, 0.001000)
    sigma2_u2_${i+1} <- 1 / tau_u2_${i+1} 
    % endfor
    % for j in range(0, numfacts):
    % if not context['constr_fact' + str(j+1)]:
    tau_fact${j+1} ~ dgamma(0.001000, 0.001000)
    sigma2_fact${j+1} <- 1 / tau_fact${j+1}
    % else:
    tau_fact${j+1} <- ${context['tau_fact' + str(j+1)]}
    % endif
    % endfor
}
'''

    latex = r'''
<% lenbeta = 0 %>
<% numfacts = int(NumFact) %>
\begin{aligned} 
\left(
\begin{array}{l}
% for i in range(0, len(y)):
\mbox{${y[i]}}_i 
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
% for i in range(0, len(y)):
\mu_{${i}i}
% if i != len(y) -1:
\\ 
% endif
% endfor
\end{array}
\right), \Omega_e \right] \\ 
% for i in range(0, len(y)):
\mu_{${i}i} & = ${mmulttex2(x, r'\beta', 'i', lenbeta)} 
% for j in range(0, numfacts):
+ 
% if not context['constr_load' + str(j+1) + '_' + str(i+1)]:
\lambda_{${j+1},${i+1}}
% else:
${context['load' + str(j+1) + '_' + str(i+1)]}
% endif
\eta_{${j+1}i}
% endfor
<% lenbeta += len(x) %>
\\ 
% endfor
\left(
\begin{array}{l}
% for i in range(0, numfacts):
\eta_{${i+1}i}
% if i != numfacts -1:
\\ 
% endif
% endfor
\end{array}
\right) & \sim \mbox{N}
\left[ 
0
, \Omega_{\eta} \right] :
\Omega_{\eta} =
\left[\begin{array}{
% for i in range(0, numfacts):
l
% endfor
}
% for i in range(0, numfacts):
% for j in range(0, numfacts):
% if j == i:
% if not context['constr_fact' + str(j+1)]:
\Omega_{\eta_{${i+1},${j+1}}}
% else:
${context['tau_fact' + str(j+1)]}
% endif
% else:
0
% endif
% if j != numfacts - 1:
&
% else:
% if i != numfacts - 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
\\ 
% for i in range(0, lenbeta):
\beta_{${i}} & \propto 1 \\ 
% endfor
% for i in range(0, numfacts):
% if not context['constr_fact' + str(i+1)]:
\Omega_{\eta_{${i+1},${i+1}}} & \sim \Gamma(0.001000, 0.001000) \\ 
% endif
% endfor
% for j in range(0, numfacts):
% for i in range(0, len(y)):
% if not context['constr_load' + str(j+1) + '_' + str(i+1)]:
\lambda_{${j+1},${i+1}} & \propto 1 \\ 
% endif
% endfor
% endfor
\end{aligned} 

'''
    
    mlnscript = '''
% for i in range(len(y)):
MVAR 1 "${y[i]}"
% endfor
IDEN 2 "_id"
IDEN 1 "resp_indicator"

<%
import numpy
T=numpy.eye(len(y), dtype='int')
%>
% for j in range(0, len(T)):
RPAT ${' '.join([str(k) for k in T[j]])}
% for i in range(0,len(x)):
ADDT "${x[i]}"
% endfor
ADDT "_levres"
RPAT
% endfor

% for i in range(len(y)):
SETV 2 "_levres.${i + 1}"
FPAR 0 "_levres.${i + 1}"
% endfor

SMAT 2 0
<%
TT=''
for j in range(0, int(NumFact)):
    TT+= str(2)+' '
    for i in range(0, len(y)):
        if context['constr_load' + str(j+1) + '_' + str(i+1)]:
            TT+=' '+str(context['load' + str(j+1) + '_' + str(i+1)])+' '+str(int(bool(context['constr_load' + str(j+1) + '_' + str(i+1)])))+ ' '
        else:
            TT+=' '+str(0)+' '+str(int(bool(context['constr_load' + str(j+1) + '_' + str(i+1)])))+ ' '
    if context['constr_fact' + str(j+1)]:
        TT+= str(context['tau_fact' + str(j+1)]) +' '+ str(int(bool(context['constr_fact' + str(j+1)])))+' '
    else:
        TT+= str(1) +' '+ str(int(bool(context['constr_fact' + str(j+1)])))+' '

%>
FACT ${len(y)} ${int(NumFact)} 0 ${TT}
'''

    preparedata = '''
mydata = data['datafile']
for i in range(0, int(NumFact)):
    context['fact' + str(i+1)].ncols = -1*len(mydata.variables[y[0]]['data'])
            
for i in range(0, len(y)):
    context['miss'+y[i]].ncols = -1*len(mydata.variables[y[i]]['data'])
'''
