# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template1LevInter(Template):
    'Fits 1-level models for Normal, Poisson or binomal responses, allowing the user to specify interactions between explanatory variables.'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal', 'Poisson', 'Binomial', 'Probit', 'Logit', 'Complementary log-log', 'Interactions', 'MLwiN:point & click']
    engines = ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS', 'MLwiN_MCMC', 'MLwiN_IGLS', 'R_MASS', 'R_MCMCglmm', 'Stata_model','SPSS_model', 'SAS_model']

    inputs = '''
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Poisson'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')

x = DataMatrix('Explanatory variables: ')
NumInter = Integer('Number of interaction terms: ')
for i in range(0, int(NumInter)):
    context['Inter' + str(i)] = DataMatrix('Enter terms for interaction ' + str(i + 1) + ': ')

if D == 'Normal':
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)
    sigma2 = ParamScalar(modelled = False)
beta = ParamVector(parents=[x], as_scalar=True)
beta.ncols += int(NumInter)
deviance = ParamScalar(modelled = False)
'''
       
    model = '''
model{
    for (i in 1:length(${y})) {
        ${y}[i] ~ \\
        % if D == 'Normal':
dnorm(mu[i], tau)
        mu[i] <- \\
        % endif
        % if D == 'Binomial':
dbin(p[i], ${n}[i])
#dbern(p[i])
        ${link}(p[i]) <- 
        % endif
        % if D == 'Poisson':
dpois(p[i])
        ${link}(p[i]) <- \\
        % if offset:
        ${n}[i] +\\
        % endif
        % endif        
<% first = True %>\\
        % for i in range(0, len(x)):
        % if first == False:
        + \\
        % endif
${x[i]}[i] * beta_${i}
<% first = False %>\\
        % endfor
        % for i in range(0, int(NumInter)):
        + \\
        % for j in range(0, len(context['Inter' + str(i)].name)):
${context['Inter' + str(i)].name[j]}[i] * \\
        % endfor
beta_${len(x) + i}
        % endfor
    }
    
    # Priors
    % for i in range(0, beta.ncols):
    beta_${i} ~ dflat()
    % endfor
    % if D == 'Normal':
    tau ~ dgamma(0.001000, 0.001000)
    sigma2 <- 1 / tau
    sigma <- 1 / sqrt(tau)
    % endif
}
'''

    latex = r'''
\begin{align}
%if D == 'Normal':
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_i & \sim \mbox{Binomial}(\mbox{${n}}_i, \pi_i) \\ 
\mbox{${link}}(\pi_i) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_i & \sim \mbox{Poisson}(\pi_i) \\ 
\mbox{${link}}(\pi_i) & =
%if offset:
\mbox{${n}}_i +
%endif
%endif
 <% numbervariables = 0 %>
 <% firstterm = True %>
%for i in range(0, len(x)):
 %if numbervariables >= 5:
  \\
  & \quad +
  <% firstterm = True %>
  <% numbervariables = 0 %>
 %endif
 %if firstterm == True:
  \beta_{${i}} \mbox{${x[i]}}_i
 %endif
 %if firstterm == False:
  + \beta_{${i}} \mbox{${x[i]}}_i
 %endif
 <% firstterm = False %>
 <% numbervariables = numbervariables + 1 %>
%endfor
%for i in range(0, int(NumInter)):
  %if numbervariables >= 5:
   \\ 
   <% firstterm = True %>
   <% numbervariables = 0 %>
  %endif
  %if firstterm == True:
  & \quad + \beta_{${len(x) + i}}
  %endif
  %if firstterm == False:
  + \beta_{${len(x) + i}}
  %endif
  <% firstterm = False %>
  <% firstinterm = True %>
%for j in range(0, len(context['Inter' + str(i)].name)):
  %if firstinterm == False:
    \times
  %endif
  <% firstinterm = False %>
\mbox{${context['Inter' + str(i)].name[j]}}
<% numbervariables = numbervariables + 1 %>
%endfor
_i
%endfor
 \\ 
%if str(Engine) in ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS' , 'MLwiN_MCMC', 'R_MCMCglmm' ]:
\beta_k & \propto 1, \qquad k \in \{0,\ldots,${len(x)+int(NumInter)-1}\} \\ 

%if D == 'Normal':
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
%endif
%endif
\end{align} 
'''

    mlnscript = '''
RESP "${y}"
IDEN 1 "_id"
% if D == 'Binomial':
RDISt 1 0
DOFFs 1 "${n}"
% if link == 'logit':
LFUN 0
% elif link == 'probit':
LFUN 1
% elif link == 'cloglog':
LFUN 2
% endif
% for p in range(len(x)):
ADDT "${x[p]}"
% endfor
% for p in range(NumInter):
ADDT \\
% for q in range(len(context['Inter' + str(p)])):
"${context['Inter'+ str(p)][q]}" \\
% endfor
% endfor
% elif D == 'Normal':
ADDT "_levres"
SETV 1 "_levres"
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% for p in range(NumInter):
ADDT \\
% for q in range(len(context['Inter' + str(p)])):
"${context['Inter'+ str(p)][q]}" \\
% endfor
% endfor
% elif D == 'Poisson':
RDISt 1 1
LFUN 3
% if offset:
DOFFs 1 "${n}"
% endif
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% for p in range(NumInter):
ADDT \\
% for q in range(len(context['Inter' + str(p)])):
"${context['Inter'+ str(p)][q]}" \\
% endfor
% endfor
% endif
'''

    latexMLwiN = r'''
\begin{aligned}
&\text{Select }\bold{Open}\text{ }\bold{Sample}\text{ } \bold{Worksheet} \text{ or }\bold{Open}\text{ } \bold{Worksheet} \text{ from the }\bold{File}\text{ menu.} \\ 
&\text{Select }\bold{datafile.dta}.\\ 
&\text{Select }\bold{Equations}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on } \mathbf{y}\text{ (either of the }\mathbf{y}\text{ symbols shown will do).}\\ 
&\text{In the } \mathbf{y}\text{ list, select }\bold{${y}}\text{.}\\ 
&\text{Select }\mathbf{1}-\mathbf{i}\text{ as the number of levels.}\\ 
&\text{Select }\bold{${y}}\text{ as }\bold{level}\text{ } \mathbf{1(i)}.\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
%if D == 'Binomial':
&\text{Click on the N and instead choose }\bold{Binomial}\text{ from the list.}\\ 
%if link == 'logit':
&\text{Click on the }\bold{Done}\text{ button.}\\ 
%endif
%if link == 'probit':
&\text{Choose }\bold{probit}\text{ from the }\bold{link}\text{ }\bold{function}\text{ list and click on the  }\bold{Done}\text{ button.}\\  
%endif
%if link == 'cloglog':
&\text{Choose }\bold{cloglog}\text{ from the }\bold{link}\text{ }\bold{function}\text{ list and click on the }\bold{Done}\text{ button.}\\  
%endif
&\text{Click on the red } n_{i}.\\ 
&\text{Select }\bold{${n}}\text{ and click on the }\bold{Done}\text{ button.}\\ 
% endif
% if D== 'Normal':
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
&\text{In the }\bold{Equations}\text{ window click on }\bold{\_levres}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{i})\bold{\_id}\text{ tick box.}\\ 
% endif
% if D== 'Poisson':
&\text{Click on the N and select }\bold{Poisson}\text{ from the popup list and click on the }\bold{Done}\text{ button.}\\ 
% if offset:
&\text{Click on the }\pi_{i}\text{ and from the }\bold{offset}\text{ window that appears select }\bold{${n}}.\\ 
% endif
% endif
% for p in range(len(x)):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{${x[p]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor
% for p in range(NumInter):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Choose }\bold{order}\text{ to be ${len(context['Inter' + str(p)].name)-1}}.\\  
% for q in range(len(context['Inter' + str(p)])):
&\text{Add }\bold{${context['Inter' + str(p)].name[q]}}\text{ as a variable}.\\    
% endfor
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor
'''       
     
    rscript = '''
formula <- %{y} ~ ${Rmmult(x)}
% for i in range(int(NumInter)):
update(formula, . ~ . + ${":".join(context['Inter' + str(i)]})
% endfor
update(formula, . ~ ., -1)

% if Rpackage == 'MASS':
% if D == 'Poisson':
% if offset:
update(formula, . ~ . , offset=${n})
% endif
% endif
% if D == 'Normal':
<% Rfamily='gaussian(identity)' %>
% elif D == 'Binomial':
<% Rfamily='binomial('+link+')' %>
denom = sum(mydata$${n}) / length(mydata$${n})
if (denom > 1) {
##binomial data
mydata$aY <- mydata$${n} - mydata$${y}
formula <- cbind(${y}, aY) ~ ${Rmmult(x)}
% for i in range(int(NumInter)):
update(formula, . ~ . + ${":".join(context['Inter' + str(i)]})
% endfor
update(formula, . ~ ., -1)
}
% endif
% elif D == 'Poisson':
<% Rfamily='poisson(log)' %>
% else:
<% Rfamily=D %>
% endif
myModel<-glm(formula,data=mydata,family=${Rfamily})
summary(myModel)
svg("ResivsFitted.svg")
plot(myModel,1)
dev.off()
svg("qqNorm.svg")
plot(myModel,2)
dev.off()
% endif

% if Rpackage == 'MCMCglmm':
<% nparams = len(x) %>
<% nparams += int(NumInter) %>
% if D == 'Poisson':
% if offset:
<% nparams += 1 %>
% endif
% endif

% if D == 'Normal':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),R=list(V=1,nu=0.002))
myModel<-MCMCglmm(formula,family="gaussian",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% elif D == 'Binomial':
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),R=list(V=1,fix=1))
% if link == 'logit':
<% denom=sum(data[n])/len(data[n]) %>
% if denom == 1:
myModel<-MCMCglmm(formula,family="categorical",prior=myprior,burnin=${burnin},nitt=${iterations+burnin},thin=${thinning},data=mydata)
% endif
% if denom > 1:
myData$aY<-myData$${n}- myData$${y}
denom = sum(mydata$${n}) / length(mydata$${n})
if (denom > 1) {
##binomial data
mydata$aY <- mydata$${n} - mydata$${y}
formula <- cbind(${y}, aY) ~ ${Rmmult(x)}
% for i in range(int(NumInter)):
update(formula, . ~ . + ${":".join(context['Inter' + str(i)]})
% endfor
update(formula, . ~ ., -1)
}
myModel<-MCMCglmm(formula,family="multinomial2",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% elif link == 'probit':
myModel<-MCMCglmm(formula,family="ordinal",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% else:
<% print "This link is not yet implemented, choose another engine" %>
% endif
% endif
% elif D == 'Poisson':
% if offset:
update(formula, . ~ . + ${n}
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})),R=list(V=1,fix=1))\n'      
% if offset:
myprior$B$V[${nparams},${nparams}]<-1e-6\n'
myprior$B$mu[${nparams}]<-1\n'
% endif
myModel<-MCMCglmm(formula,family="poisson",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% endif
% else:
<% print 'Distribution not implemented yet, choose another engine:' %>
% endif
summary(myModel)

% if D == 'Binomial':
% if link == 'logit':
c<-((16*sqrt(3))/(15*pi))^2
myModel$Sol<-myModel$Sol/sqrt(1+c*myModel$VCV[,"units"])
myModel$VCV<-myModel$VCV/(1+c*myModel$VCV[,"units"])
% elif link == 'probit':
c<-1
myModel$Sol<-myModel$Sol/sqrt(1+c*myModel$VCV[,"units"])
myModel$VCV<-myModel$VCV/(1+c*myModel$VCV[,"units"])
% endif
% endif
myModelOut<-cbind(myModel$Sol,myModel$VCV)
myModelOut<-mcmc(myModelOut)

<% ind=divmod(nparams,3) %>
% for i in range(ind[0]):
<% svgout='DiagPlots'+str(i+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${i}):(3*${i+1})])
dev.off()
% endfor
% if ind[1] > 0:
<% svgout='DiagPlots'+str(ind[0]+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${ind[0]}):${nparams}])
dev.off() 
% endif
colnames(myModelOut)[which(colnames(myModelOut)=="units")]<-"sigma^2"
write.dta(data.frame(myModelOut),file="chains.dta")
## export DIC
dic <- myModel$DIC
write.dta(data.frame(dic), file = "dic.dta")
% endif
'''
        
    statascript = '''
% if D == 'Normal':
local family gaussian
local link identity
% elif D == 'Binomial':
local family binomial
quietly summarize ${n}
local denom = `r(sum)' / `r(N)'
if `denom' > 1 {
local family `family' ${n}
% endif
local link ${link}
% elif D == 'Poisson':
local family poisson
local link log
% else:
% endif

/// Create interaction terms
gen \\
<% dummy = '' %>\\
% for p in range(int(NumInter)):
<% dummy+='Inter_'+str(p)+'=' %>\\
% for q in range(len(context['Inter'+str(p)].name)-1):
<% dummy+=context['Inter'+str(p)].name[q] + '*' %>\\
% endfor
<% dummy+=context['Inter'+str(p)].name[len(context['Inter'+str(p)].name)-1] %>\\
% endfor
${dummy}

/// Fit model
glm ${y} \\
% for p in x:
${p} \\
% endfor
% for p in range(int(NumInter)):
Inter_${p} \\
% endfor
, family(`family') link(`link') \\
% if D == 'Poisson':
% if offset:
offset(${n}) \\
% endif
% endif
noconstant

predict yhat, mu
predict ehat, response
predict rhat, response standardized
scatter ehat yhat
graph export "ResivsFitted.png", replace
egen rank=rank(rhat)
gen nscore=invnormal(rank/(_N+1))
scatter rhat nscore
graph export "QQ_Plot.png", replace
'''

    spssscript = '''
GENLIN \\
% if D == 'Binomial':
${y} OF ${n} with \\
% else:
${y} with \\
% endif
% for p in x:
${p} \\
% endfor
/MODEL \\
% for p in x:
${p} \\
% endfor
% for p in range(0, int(NumInter)):
% for q in range(0, len(context['Inter' + str(p)])-1):
${context['Inter' + str(p)][q]}*\\
% endfor
${context['Inter' + str(p)][len(context['Inter' + str(p)])-1]} \\
% endfor
INTERCEPT=NO \\
% if D == 'Normal':
DISTRIBUTION=NORMAL \\
% elif D == 'Binomial':
DISTRIBUTION=BINOMIAL LINK=${link} \\
% elif D == 'Poisson':
% if offset:
OFFSET=${n} \\
% endif
DISTRIBUTION=POISSON LINK=LOG \\
% endif
/CRITERIA ANALYSISTYPE=3(LR) LCONVERGE=1E-4(ABSOLUTE).
'''

    sasscript = '''
% if D == 'Normal':
proc glm data=data;
model ${y} = \\
% for p in x:
${p} \\
% endfor
% for p in range(0, int(NumInter)):
% for q in range(0, len(context['Inter' + str(p)])-1):
${context['Inter' + str(p)][q]}*\\
% endfor
${context['Inter' + str(p)][len(context['Inter' + str(p)])-1]} \\
% endfor
/noint;
run;
% endif

% if D == 'Poisson':
proc Genmod data=data;
model ${y} = \\
% for p in x:
${p} \\
% endfor
% for p in range(0, int(NumInter)):
% for q in range(0, len(context['Inter' + str(p)])-1):
${context['Inter' + str(p)][q]}*\\
% endfor
${context['Inter' + str(p)][len(context['Inter' + str(p)])-1]} \\
% endfor
/ dist=poi link =log \\
%if offset:
offset=${n} \\
%endif
noint;
run;
% endif

% if D == 'Binomial':
proc logistic data=data;
model ${y}/${n} = \\
% for p in x:
${p} \\
% endfor
% for p in range(0, int(NumInter)):
% for q in range(0, len(context['Inter' + str(p)])-1):
${context['Inter' + str(p)][q]}*\\
% endfor
${context['Inter' + str(p)][len(context['Inter' + str(p)])-1]} \\
% endfor
/ link=${link} noint;
run;
% endif
'''
