# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevBlock(Template):       
    'Fits 2-level random intercept Normal models using a multivariate step for the fixed effects.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'MLwiN:point & click' ]
    engines = ['eStat', 'WinBUGS', 'OpenBUGS', 'MLwiN_MCMC', 'MLwiN_IGLS']

    inputs = '''
y = DataVector('Response: ')
L2ID = IDVector('Level 2 ID: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
storeresid = Boolean('Store level 2 residuals?')
beta = ParamVector(parents=[x], as_scalar=True, customstep=True)
tau = ParamScalar()
sigma = ParamScalar(modelled = False)
if storeresid:
    u = ParamVector(parents=[L2ID], as_scalar=False)
else:
    u = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
tau_u = ParamScalar()
sigma_u = ParamScalar(modelled = False)
deviance = ParamScalar(modelled=False)
'''

    model = '''
model {
    for (i in 1:length(${y})) {
        ${y}[i] ~ \\
dnorm(mu[i], tau)
        mu[i] <- \\
${mmult(x, 'beta', 'i')} + u[${L2ID}[i]]
    }

    for (j in 1:length(u)) {
        u[j] ~ dnorm(0, tau_u)
    }
    
    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
    
    tau ~ dgamma(0.001000, 0.001000)
    sigma <- 1 / sqrt(tau)
    tau_u ~ dgamma(0.001000, 0.001000)
    sigma_u <- 1 / sqrt(tau_u)
}
'''

    latex = r'''
\begin{aligned} 
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
${mmulttex(x, r'\beta', 'i')} + u_{\mbox{${L2ID}}[i]}\\ 
u_{\mbox{${L2ID}}[i]} & \sim \mbox{N}(0,\sigma^2_u) \\ 
%if str(Engine) in ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS' , 'MLwiN_MCMC', 'R_MCMCglmm' ]:
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
\tau_u & \sim \Gamma (0.001,0.001) \\ 
\sigma_u^2 & = 1 / \tau_u
%endif
\end{aligned} 
'''

    preccode = '''
            {
                bool fail = false;
                static RectMatrix xtxchol(${len(x)}, ${len(x)});
                // Setting up constant terms for beta step
                if (runstate == 0) {
                    xtxchol = matrix_cholesky(mat_x.T() * mat_x, false, fail);
                }
                // Multivariate step for beta
                RectMatrix ydiff(mat_y);
                for (int i = 0; i < ydiff.rows(); i++) {
                    ydiff(i, 0) -= double(u[int(${L2ID}[i])]);
                }
                RectMatrix xty = mat_x.T() * ydiff;
                RectMatrix mean = matrix_cholsolve(xtxchol, xty);
                DiagMatrix taudiag = identity(${len(x)}) / tau;
                SymMatrix variance = matrix_cholsolve(xtxchol, taudiag);
                mat_beta = dmultnormal(mean, variance, fail);
             }
         '''

    mlnscript = '''
RESP "${y}"
IDEN 2 "${L2ID}"
IDEN 1 "_id"
ADDT "_levres"
SETV 2 "_levres"
SETV 1 "_levres"
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
'''

    latexMLwiN = r'''
\begin{aligned}
&\text{Select }\bold{Open}\text{ }\bold{Sample}\text{ } \bold{Worksheet} \text{ or }\bold{Open}\text{ } \bold{Worksheet} \text{ from the }\bold{File}\text{ menu.} \\ 
&\text{Select }\bold{datafile.dta}.\\ 
&\text{Select }\bold{Equations}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on } \mathbf{y}\text{ (either of the }\mathbf{y}\text{ symbols shown will do).}\\ 
&\text{In the } \mathbf{y}\text{ list, select }\bold{${y}}\text{.}\\ 
&\text{In the }\bold{N}\text{ }\bold{levels}\text{ list, select } \mathbf{2}-\mathbf{ij}.\\ 
&\text{In the }\bold{level} \text{ }\mathbf{2(j):}\text{ list, select } \bold{${L2ID}}.\\  
&\text{In the }\bold{level} \text{ }\mathbf{1(i):}\text{ list, select } \bold{\_id}.\\  
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% for p in range(len(x)):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{${x[p]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
&\text{In the }\bold{Equations}\text{ window click on }\bold{\_levres}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{j})\bold{${L2ID}}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{i})\bold{\_id}\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
'''

    latexMLwiNextra = r'''
&\text{Select }\bold{MCMC}/\bold{MCMC}\text{ }\bold{options}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on }\bold{Structured}\text{ }\bold{MVN}.\\ 
&\text{Click on the }\bold{Done}\text{ button on the }\bold{MCMC}\text{ }\bold{options}\text{ window.}\\ 
'''
