# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevBlock(Template):       
    'Fits 2-level random intercept Normal models using a multivariate step for the fixed effects.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level']
    engines = ['CustomC']

    inputs = '''
y = DataVector('Response: ')
L2ID = IDVector('Level 2 ID: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
storeresid = Boolean('Store level 2 residuals?')
if storeresid:
    u = ParamVector(parents=[L2ID], as_scalar=False)
else:
    u = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
tau = ParamScalar()
sigma = ParamScalar(modelled = False)
tau_u = ParamScalar()
sigma_u = ParamScalar(modelled = False)
deviance = ParamScalar(modelled = False)
'''
    latex = r'''
\begin{aligned} 
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
${mmulttex(x, r'\beta', 'i')} + u_{\mbox{${L2ID}}[i]}\\ 
u_{\mbox{${L2ID}}[i]} & \sim \mbox{N}(0,\sigma^2_u) \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
\tau_u & \sim \Gamma (0.001,0.001) \\ 
\sigma_u^2 & = 1 / \tau_u
\end{aligned} 
'''

    customccode = '''
            {
                bool fail = false;
                static RectMatrix xtxchol(${len(x)}, ${len(x)});
                // Setting up constant terms for beta step
                if (runstate == 0) {
                    xtxchol = matrix_cholesky(mat_x.T() * mat_x, false, fail);
                }

                for (int i = 0; i < mat_u.rows(); i++) {
                    double sumy = 0.0;
                    for (int id_k = 0; id_k < idmapsizes_school[i]; id_k++) {
                        int k = idmap_school[i][id_k];
                        sumy += mat_y(k, 0);
                    }
                    double mean = sumy;
                    for (int j = 0; j < mat_beta.rows(); j++) {
                        double sumx = 0.0;
                        for (int id_k = 0; id_k < idmapsizes_school[i]; id_k++) {
                            int k = idmap_school[i][id_k];
                            sumx += mat_x(k, j);
                        }
                        mean -= mat_beta(j, 0) * sumx;
                    }
                    
                    double du = tau_u + tau * idmapsizes_school[i];
                    mat_u(i, 0) = dnorm((tau * mean) / du, du);
                }

                tau_u = dgamma(0.001 + 0.5 * mat_u.rows(), 0.001 + 0.5 * (mat_u.T() * mat_u)(0, 0));
                sigma_u = 1/tau_u;

                // Multivariate step for beta
                RectMatrix ydiff(mat_y);
                for (int i = 0; i < ydiff.rows(); i++) {
                    ydiff(i, 0) -= double(u[int(${L2ID}[i])]);
                }
                RectMatrix xty = mat_x.T() * ydiff;
                RectMatrix mean = matrix_cholsolve(xtxchol, xty);
                DiagMatrix taudiag = identity(${len(x)}) / tau;
                SymMatrix variance = matrix_cholsolve(xtxchol, taudiag);
                mat_beta = dmultnormal(mean, variance, fail);

                // Note: ydiff is y - u from above
                RectMatrix yxb = ydiff - mat_x * mat_beta;
                double sum0 = (yxb.T() * yxb)(0,0);

                tau = dgamma(0.001 + 0.5 * length(${y}), 0.001 + 0.5 * sum0);
                sigma = 1/tau;
                deviance = (2 * (((tau * sum0) / 2) + (0.5 * (log(3.14159265) - log(tau)) * length(${y})) + (0.346573590279973 * length(${y})))); // Last term is log(sqrt(2))
             }
'''

    diccode = '''
        RectMatrix ydiff(mat_y);
        for (int i = 0; i < ydiff.rows(); i++) {
            ydiff(i, 0) -= double(u[int(${L2ID}[i])]);
        }
        RectMatrix yxb = ydiff - mat_x * mat_beta;
        double sum0 = (yxb.T() * yxb)(0,0);
        deviance = (2 * (((tau * sum0) / 2) + (0.5 * (log(3.14159265) - log(tau)) * length(${y})) + (0.346573590279973 * length(${y})))); // Last term is log(sqrt(2))
'''
