# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevComplex(Template):       
    'Fits 2-level models for Normal or t-distributions, allowing heteroskedasticity in the response. NB by default WinBUGS has adapting phase of 4000 iterations for MH (see <a href="http://voteview.com/WINBUGSmanual_2.pdf" target="blank">Spiegelhalter et al 2003</a>), so suggest this as minimum for burn-in.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'T', 'Complex level 1', 'MLwiN:point & click' ]
    engines = ['eStat', 'OpenBUGS', 'JAGS', 'MLwiN_IGLS', 'MLwiN_MCMC']

    inputs = '''

def combs(s):
    out = []
    for i in range(0, len(s)):
        for j in range(0, i + 1):
            out.append(s[i] + '*' + s[j])
    return out

y = DataVector('Response: ')
L2ID = IDVector('Level 2 ID: ')
D = Text('Specify distribution: ', ['Normal', 't'])
x = DataMatrix('Explanatory variables: ', allow_cat = True)
complexvars = combs(x)
for i in range(0, len(complexvars)):
    context['inc' + str(i)] = Boolean('Include variation for ' + complexvars[i] + '? ')

storeresid = Boolean('Store level 2 residuals?')
if storeresid:
    u = ParamVector(parents=[L2ID], as_scalar=False)
else:
    u = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
beta = ParamVector(parents=[x], as_scalar=True)
tau_u = ParamScalar()
sigma2_u = ParamScalar(modelled = False)
for i in range(0, len(complexvars)):
    if context['inc' + str(i)]:
        context['alpha' + str(i)] = ParamScalar()
deviance = ParamScalar(modelled = False)
'''

    model = '''
model {
    for (i in 1:length(${y})) {
            ${y}[i] ~ dnorm(mu[i], tau[i])
            mu[i] <- ${mmult(x, 'beta', 'i')} + u[${L2ID}[i]]
            tau[i] <- 1/( \\
<% complexvars = combsi(x, 'i') %> \\
<% first = True %> \\
            % for i in range(0, len(complexvars)):
            % if context['inc' + str(i)]:
            % if first != True:
+ \\
            % endif
alpha${i} * ${complexvars[i]} \\
<% first = False %> \\
            % endif
            % endfor
)            
    }

    for (j in 1:length(u)) {
        % if D == 't':
        u[j] ~ dt(0, tau_u, df)
        % else:
        u[j] ~ dnorm(0, tau_u)
        % endif
    }
    
    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
    
    % if D == 't':
    df <- 8
    % endif
    
    tau_u ~ dgamma(0.001000, 0.001000)
    sigma2_u <- 1 / tau_u
    % for i in range(0, len(complexvars)):
    % if context['inc' + str(i)]:
    alpha${i} ~ dflat()
    % endif
    % endfor    
}
'''

    latex = r'''
<%!
def combslat(s, index):
        out = []
        for i in range(0, len(s)):
            for j in range(0, i + 1):
                if s[j]==s[i]:
                    out.append(r'\mbox{' + s[i] + '}_' + index +  '^2')
                else :
                    out.append(r'2 \mbox{'+s[i]+'}_'+index +r'\mbox{'+s[j]+'}_'+index)
        return out 
%>
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma_i^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} + u_{\mbox{${L2ID}}[i]}\\ 
%if D=='Normal':
u_{\mbox{${L2ID}}[i]} & \sim \mbox{N}(0,\sigma^2_u) \\ 
%endif
%if D=='t':
u_{\mbox{${L2ID}}[i]} & \sim \mbox{t}(0,\tau_u,\df) \\ 
%endif
\sigma_i^2 & =  \\
   <% complexvars = combslat(x, 'i') %>      
 <% first = True %>
        % for i in range(0, len(complexvars)):
        % if context['inc' + str(i)]:
        % if first != True:
        +
        % endif
        ${complexvars[i]} \alpha_${i} 
        <% first = False %>
        % endif
        % endfor
\\ 
%if str(Engine) in ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS' , 'MLwiN_MCMC', 'R_MCMCglmm' ]:
\beta_0 & \propto 1 \\ 
%for i in range(1, len(x)):
\beta_{${i}} & \propto 1 \\ 
%endfor
%for i in range(0, len(complexvars)):
    % if context['inc'+str(i)]:
    \alpha_{${i}} & \propto 1 \\ 
    % endif
%endfor
\tau_u & \sim \Gamma (0.001,0.001) \\ 
\sigma_u^2 & = 1 / \tau_u
%if D=='t':
\df & = 8
%endif
%endif
\end{aligned} 
'''
    
    mlnscript = '''
RESP "${y}"
IDEN 2 "${L2ID}"
IDEN 1 "_id"
% if D == 'Normal':
ADDT "_levres"
SETV 2 "_levres"
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
SETV 1 "${x[i]}"
% endfor
<%
def combs(s):
    out = []
    for i in range(0, len(s)):
        for j in range(0, i + 1):
            out.append(s[i] + '*' + s[j])
    return out

def combs1(a,x):
    out = []
    count=0
    for i in range(0, len(x)):
        for j in range(0, i + 1):
            if a[count]==False:
                out.append(x[i])
                out.append(x[j])
            count+=1
    return out

import numpy
complexvars = combs(x)
T=[]
for i in range(0, len(complexvars)):
    T.append(bool(context['inc' + str(i)]))

TT=combs1(T,x)
TT1=[]
for i in range(0,len(TT),2):
    TT1.append(TT[i])
TT2=[]   
for i in range(1,len(TT),2):
    TT2.append(TT[i])

%>

% for i in range(len(TT1)):
CLRE 1 "${TT1[i]}" "${TT2[i]}"
% endfor
% else:
print 'Distribution not recognised'
% endif

'''

    latexMLwiN = r'''
<%
def combs(s):
    out = []
    for i in range(0, len(s)):
        for j in range(0, i + 1):
            out.append(s[i] + '*' + s[j])
    return out

def combs1(a,x):
    out = []
    count=0
    for i in range(0, len(x)):
        for j in range(0, i + 1):
            if a[count]==False:
                out.append(x[i])
                out.append(x[j])
            count+=1
    return out

import numpy
complexvars = combs(x)
T=[]
for i in range(0, len(complexvars)):
    T.append(bool(context['inc' + str(i)]))

TT=combs1(T,x)
TT1=[]
for i in range(0,len(TT),2):
    TT1.append(TT[i])
TT2=[]   
for i in range(1,len(TT),2):
    TT2.append(TT[i])

%>

\begin{aligned}
&\text{Select }\bold{Open}\text{ }\bold{Sample}\text{ } \bold{Worksheet} \text{ or }\bold{Open}\text{ } \bold{Worksheet} \text{ from the }\bold{File}\text{ menu.} \\ 
&\text{Select }\bold{datafile.dta}.\\ 
&\text{Select }\bold{Equations}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on } \mathbf{y}.\\ 
&\text{In the } \mathbf{y}\text{ list, select }\bold{${y}}\text{.}\\ 
&\text{In the }\bold{N}\text{ }\bold{levels}\text{ list, select } \mathbf{2}-\mathbf{ij}.\\ 
&\text{In the }\bold{level} \text{ }\mathbf{2(j):}\text{ list, select } \bold{${L2ID}}.\\  
&\text{In the }\bold{level} \text{ }\mathbf{1(i):}\text{ list, select } \bold{\_id}.\\  
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% for p in range(len(x)):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{${x[p]}}\text{ from the variable list.}\\ 
&\text{Add the tick in the }(\mathbf{i})\bold{${x[p]}}\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
&\text{In the }\bold{Equations}\text{ window click on }\bold{\_levres}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{j})\bold{${L2ID}}\text{ tick box.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% for i in range(len(TT1)):
&\text{Click on the position for }\bold{${TT1[i]}}/\bold{${TT2[i]}}\text{ in the level 1 variance matrix}.\\ 
&\text{A message asking if you want to remove }\bold{${TT1[i]}}/\bold{${TT2[i]}}\text{ should appears}.\\ 
&\text{Choose }\bold{yes}\text{ to remove this term}.\\ 
% endfor
'''
