# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateXC(Template):
    'Fits random intercept models with two correlated classifications (nested or crossed). Allows for Normal, Poisson or binomial distributions, and a choice of uniform or Wishart priors.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'N-Level', 'Normal', 'Poisson', 'Binomial', 'Probit', 'Logit', 'Complementary log-log', 'Correlated classifications' ]
    engines = ['eStat']

    inputs = '''
y = DataVector('Response: ')
C1 = IDVector('Classification 1: ')
C2 = IDVector('Classification 2: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Poisson'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
storeresid = Boolean('Store residuals?')

priors = Text('Priors: ', ['Uniform', 'Wishart'])
if priors == 'Wishart':
    R = Text('R matrix: ')
    v = Integer('Degrees of Freedom:')

if D == 'Normal':
    tau = ParamScalar()
    sigma = ParamScalar(modelled = False)
beta = ParamVector(parents=[x], as_scalar=True)
omega = ParamMatrix(modelled = False, customstep=True)
omega.size = 2
d = ParamMatrix(customstep=True)
d.size = 2

if storeresid:
    u1 = ParamVector(parents=[C1], as_scalar=False)
    u2 = ParamVector(parents=[C2], as_scalar=False)
else:
    u1 = ParamVector(parents=[C1], as_scalar=False, monitor=False)
    u2 = ParamVector(parents=[C2], as_scalar=False, monitor=False)

deviance = ParamScalar(modelled=False)
'''
        
    model = '''
model {
            for (i in 1:length(${y})) {
                ${y}[i] ~
                % if D == 'Normal':
                dnorm(mu[i], tau)
                mu[i] <-
                % endif
                % if D == 'Binomial':
                dbin(p[i], ${n}[i])
                ${link}(p[i]) <-
                % endif
                % if D == 'Poisson':
                dpois(p[i])
                ${link}(p[i]) <- 
                % if offset:
                ${n}[i] +
                % endif
                % endif    
                ${mmult(x, 'beta', 'i')} + u1[${C1}[i]] + u2[${C2}[i]]
            }
    
    for (i1 in 1:length(u1)) {
        dummy[i1] ~ dnormal2a(u1[i1], u2[i1], 0, 0, d[0], d[1], d[2])
        u1[i1] ~ dflat()
        u2[i1] ~ dflat()
        dummy2[i1] ~ ddummy(dummy[i1])
    }
    
    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
    % if D == 'Normal':
    tau ~ dgamma(0.001000, 0.001000)
    sigma <- 1 / sqrt(tau)
    % endif
}       
'''

    latex = r'''
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
${mmulttex(x, r'\beta', 'i')} 
+ u^{(1)}_{\mbox{${C1}}[i]} + u^{(2)}_{\mbox{${C2}}[i]}
\\ 
\left(
\begin{array}{l}
u^{(1)}_{\mbox{${C1}}[i]}  \\ 
u^{(2)}_{\mbox{${C2}}[i]} 
\end{array}
\right) & \sim \mbox{N}
\left[ \left(
\begin{array}{l}
0 \\ 
0
\end{array}
\right), \Omega_{u} \right] \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
%if D == 'Normal':
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
%endif
% if context['priors'] == 'Wishart':
\Omega_u & \sim \mbox{inverse Wishart}_{2}[${context['v']}*R_u,${context['v']}],R_u =
\left[\begin{array}{
ll
}
<% lenR = 0%>
% for i in range(0, 2):
% for j in range(0, 2):
% if j <= i:
${context['R'].name[lenR]}
<% lenR += 1 %>
% endif
% if j != 1:
&
% else:
% if i != 1:
\\ 
% endif
% endif
% endfor
% endfor
\end{array}
\right]
% endif
% if context['priors'] == 'Uniform':
\Omega_u & \propto 1
% endif
\\ 
\end{aligned} 
'''

    preccode = '''
            {
                // Note currently using a uniform prior for variance matrix
                SymMatrix sb(2);
                for(int i = 0; i < length(u1); i++) {
% for j in range(0, 2):
% for k in range(j, 2):
                    sb(${j},${k}) += double(u${j + 1}[i]) * double(u${k + 1}[i]);
% endfor
% endfor
                }
% if priors == 'Uniform':
                int vw = length(u1)-3;
                if (runstate == 0) {
% for j in range(0, 2):
                    sb(${j},${j}) += 0.0001;
% endfor
                }
% endif
% if priors == 'Wishart':
                int vw = length(u1) + ${v};
% endif

% if priors == 'Wishart':
<%
import numpy
Rmat = numpy.empty([2, 2])
count = 0 
for j in range(0, 2):
    for k in range(0, j + 1):
        Rmat[j, k] = float(R.name[count]) 
        Rmat[k, j] = Rmat[j, k]
        count += 1
%>

<% count = 0 %>
% for j in range(0, 2):
% for k in range(0, 2):
                sb(${j},${k}) += ${str(Rmat[j][k] * float(v))};
% endfor
% endfor
% endif

                matrix_sym_invinplace(sb);

		bool fail = false;
		mat_d = dwishart(vw, sb, fail);
		mat_omega = matrix_sym_inverse(mat_d, fail);
            }
'''
