# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevMod(Template):       
    'Fits 2-level random intercept models for Normal, Poisson, binomial or t-distributions (Note: not all engines can currently model t-distribution). For SPSS, model distributions other than Normal are only supported by SPSS version 19 and above.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Poisson', 'Binomial', 'T', 'Probit', 'Logit', 'Complementary log-log', 'MLwiN:point & click']
    engines = ['eStat','WinBUGS','OpenBUGS','JAGS', 'MLwiN_MCMC', 'MLwiN_IGLS','R_lme4','R_INLA', 'R_MCMCglmm','Stata_model', 'SPSS_model', 'R_RStan','R_mgcv', 'SAS_model','GenStat_model']

    inputs = '''
y = DataVector('Response: ')
L2ID = IDVector('Level 2 ID: ')
D = Text('Specify distribution: ', ['Normal', 'Poisson', 'Binomial', 't'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')

storeresid = Boolean('Store level 2 residuals?')

if D == 'Normal':
    tau = ParamScalar()
    sigma2 = ParamScalar()
if D == 't':
    tau = ParamScalar()
    sigma2 = ParamScalar()
    #df = ParamScalar()
if storeresid:
    u = ParamVector(parents=[L2ID], as_scalar=False)
else:
    u = ParamVector(parents=[L2ID], as_scalar=False, monitor=False)
tau_u = ParamScalar()
sigma2_u = ParamScalar()

x = DataMatrix('Explanatory variables: ')
beta = ParamVector(parents=[x], as_scalar=True)

deviance = ParamScalar(modelled = False)
'''

    model = '''
model {
    for (i in 1:length(${y})) {
        ${y}[i] ~ \\
        % if D == 'Normal':
dnorm(mu[i], tau)
        mu[i] <- \\
        % endif
        % if D == 'Binomial':
dbin(p[i], ${n}[i])
        ${link}(p[i]) <- \\
        % endif
        % if D == 'Poisson':
dpois(p[i])
        ${link}(p[i]) <- \\
        % if offset:
${n}[i] + \\
        % endif
        % endif                   
        % if D == 't':
dnorm(mu[i], tau)
        mu[i] <- \\
        % endif            
${mmult(x, 'beta', 'i')} + u[${L2ID}[i]]
    }

    for (j in 1:length(u)) {
        % if D == 't':
        u[j] ~ dt(0, tau_u, df)
        % else:
        u[j] ~ dnorm(0, tau_u)
        % endif
    }
    
    # Priors
    % for i in range(0, x.ncols()):
    beta_${i} ~ dflat()
    % endfor
    
    % if D == 'Normal':
    tau ~ dgamma(0.001000, 0.001000)
    sigma2 <- 1 / tau
    % endif
    % if D == 't':
    tau ~ dgamma(0.001000, 0.001000)
    sigma2 <- 1 / tau
    #df ~ dunif(2, 200)
    df <- 8
    % endif
    
    tau_u ~ dgamma(0.001000, 0.001000)
    sigma2_u <- 1 / tau_u
}
'''

    latex = r'''
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
%if D == 't' :
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
${mmulttex(x, r'\beta', 'i')} + u_{\mbox{${L2ID}}[i]}\\ 
%if D=='t':
u_{\mbox{${L2ID}}[i]} & \sim \mbox{t}(0,\tau_u,\df) \\ 
%else:
u_{\mbox{${L2ID}}[i]} & \sim \mbox{N}(0,\sigma^2_u) \\ 
%endif
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
%if D == 'Normal':
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau \\ 
%endif
\tau_u & \sim \Gamma (0.001,0.001) \\ 
\sigma_u^2 & = 1 / \tau_u
%if D=='t':
\df & = 8
%endif
\end{aligned} 
'''

# Create template specific MLwiN macro/input and outputs
    
    mlnscript = '''
RESP "${y}"
IDEN 2 "${L2ID}"
IDEN 1 "_id"
% if D == 'Binomial':
RDISt 1 0
DOFFs 1 "${n}"
% if link == 'logit':
LFUN 0
% elif link == 'probit':
LFUN 1
% elif link == 'cloglog':
LFUN 2
% endif
ADDT "_levres"
SETV 2 "_levres"
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% elif D == 'Normal':
ADDT "_levres"
SETV 1 "_levres"
SETV 2 "_levres"
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% elif D == 'Poisson':
RDISt 1 1
LFUN 3
% if offset:
DOFFs 1 "${n}"
% endif
ADDT "_levres"
SETV 2 "_levres"
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% else:
print 'Distribution not recognised'
% endif
'''

    latexMLwiN = r'''
\begin{aligned}
&\text{Select }\bold{Open}\text{ }\bold{Sample}\text{ } \bold{Worksheet} \text{ or }\bold{Open}\text{ } \bold{Worksheet} \text{ from the }\bold{File}\text{ menu.} \\ 
&\text{Select }\bold{datafile.dta}.\\ 
&\text{Select }\bold{Equations}\text{ from the }\bold{Model}\text{ menu.}\\ 
&\text{Click on } \mathbf{y}\text{ (either of the }\mathbf{y}\text{ symbols shown will do).}\\ 
&\text{In the } \mathbf{y}\text{ list, select }\bold{${y}}\text{.}\\ 
&\text{In the }\bold{N}\text{ }\bold{levels}\text{ list, select } \mathbf{2}-\mathbf{ij}.\\ 
&\text{In the }\bold{level} \text{ }\mathbf{2(j):}\text{ list, select } \bold{${L2ID}}.\\  
&\text{In the }\bold{level} \text{ }\mathbf{1(i):}\text{ list, select } \bold{\_id}.\\  
&\text{Click on the }\bold{Done}\text{ button.}\\ 
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{\_levres}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
%if D == 'Binomial':
&\text{Click on the N and instead choose }\bold{Binomial}\text{ from the list.}\\ 
%if link == 'logit':
&\text{Click on the }\bold{Done}\text{ button.}\\ 
%endif
%if link == 'probit':
&\text{Choose }\bold{probit}\text{ from the }\bold{link}\text{ }\bold{function}\text{ list and click on the  }\bold{Done}\text{ button.}\\  
%endif
%if link == 'cloglog':
&\text{Choose }\bold{cloglog}\text{ from the }\bold{link}\text{ }\bold{function}\text{ list and click on the }\bold{Done}\text{ button.}\\  
%endif
&\text{Click on the red } n_{ij}.\\ 
&\text{Select }\bold{${n}}\text{ and click on the }\bold{Done}\text{ button.}\\ 
&\text{In the }\bold{Equations}\text{ window click on }\bold{\_levres}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{j})\bold{${L2ID}}\text{ tick box.}\\ 
% endif
% if D== 'Normal':
&\text{In the }\bold{Equations}\text{ window click on }\bold{\_levres}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{j})\bold{${L2ID}}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{i})\bold{\_id}\text{ tick box.}\\ 
% endif
% if D== 'Poisson':
&\text{Click on the N and select }\bold{Poisson}\text{ from the popup list and click on the }\bold{Done}\text{ button.}\\ 
% if offset:
&\text{Click on the }\pi_{i}\text{ and from the }\bold{offset}\text{ window that appears select }\bold{${n}}.\\ 
% endif
&\text{In the }\bold{Equations}\text{ window click on }\bold{\_levres}.\\ 
&\text{Remove the tick in the }\bold{Fixed}\text{ }\bold{parameter}\text{ tick box.}\\ 
&\text{Add the tick in the }(\mathbf{j})\bold{${L2ID}}\text{ tick box.}\\ 
% endif
% for p in range(len(x)):
&\text{Click the }\bold{Add}\text{ }\bold{Term}\text{ button on the tool bar.}\\ 
&\text{Select }\bold{${x[p]}}\text{ from the variable list.}\\ 
&\text{Click on the }\bold{Done}\text{ button.}\\ 
% endfor

'''
    
    rscript = '''
% if Rpackage == 'INLA':
formula <- ${y} ~ ${Rmmult(x)} + f(${L2ID}, model = "iid") - 1
% if D == 'Normal':
<% Rfamily='gaussian' %>
myModel = inla(formula,family="${Rfamily}", data = mydata)
summary(myModel)
write.dta(data.frame(myModel$summary.random$${L2ID}), file="random_${L2ID}.dta")
#tempory plot
mysvg <- function(filename = "${L2ID}Plot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.fixed.effects = F,plot.hyperparameters =F)
dev.off()
mysvg <- function(filename = paste("fixedPlot", dev.cur(), ".svg", sep=""), ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.random.effects = F,plot.hyperparameters =F)
dev.off()
dev.off()
mysvg <- function(filename = "hyperparamPlot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.random.effects = F,plot.fixed.effects = F)
dev.off()
% elif D == 'Binomial':
<% Rfamily='binomial' %>
myModel = inla(formula,family="${Rfamily}",Ntrials=${n},control.family = list(link = "${link}"), data = mydata)
summary(myModel)
write.dta(data.frame(myModel$summary.random$${L2ID}), file="random_${L2ID}.dta")
#tempory plot
mysvg <- function(filename = "${L2ID}Plot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.fixed.effects = F,plot.hyperparameters =F)
dev.off()
mysvg <- function(filename = paste("fixedPlot", dev.cur(), ".svg", sep=""), ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.random.effects = F,plot.hyperparameters =F)
dev.off()
dev.off()
mysvg <- function(filename = "hyperparamPlot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.random.effects = F,plot.fixed.effects = F)
dev.off()
% elif D == 'Poisson':
<% Rfamily='poisson' %>
##if there is an offset this needs to be added to the model
% if offset:
myModel = inla(formula,family="${Rfamily}",E=exp(${n}), data = mydata)
%else:
myModel = inla(formula,family="${Rfamily}", data = mydata)
%endif
summary(myModel)
write.dta(data.frame(myModel$summary.random$${L2ID}), file="random_${L2ID}.dta")
#tempory plot
mysvg <- function(filename = "${L2ID}Plot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.fixed.effects = F,plot.hyperparameters =F)
dev.off()
mysvg <- function(filename = paste("fixedPlot", dev.cur(), ".svg", sep=""), ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.random.effects = F,plot.hyperparameters =F)
dev.off()
dev.off()
mysvg <- function(filename = "hyperparamPlot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot.inla(myModel, plot.random.effects = F,plot.fixed.effects = F)
dev.off()
% else:
<% print 'Distribution not implemented yet, choose another engine:' %>
% endif
% endif
% if Rpackage == 'lme4':
formula <- ${y} ~ ${Rmmult(x)} - 1 + (1|${L2ID})
% if D == 'Poisson':
% if offset:
update(formula, . ~ ., offset=${n})
% endif
% endif
% if D == 'Normal':
<% Rfamily='gaussian' %>
% elif D == 'Binomial':
<% Rfamily='binomial('+str(link)+')' %>
denom = sum(mydata$${n}) / length(mydata$${n})
if (denom > 1) {
##binomial data
mydata$aY <- mydata$${n} - mydata$${y}
formula <- cbind(${y}, aY) ~ ${Rmmult(x)} - 1
}
% elif D == 'Poisson':
<% Rfamily='poisson(log)' %>
% else:
<% Rfamily=D %>
% endif

myModel<-glmer(formula,data=mydata,family=${Rfamily},nAGQ=${nAGQ})
summary(myModel)
#png("ResivsFitted.png",width=733,height=550)
#plot(myModel,1)
#dev.off()

#png("qqNorm.png",width=733,height=550)
#plot(myModel,2)
#dev.off()
% endif

% if Rpackage == 'MCMCglmm':
formula <- ${y} ~ ${Rmmult(x)} - 1
<% nparams=len(x) %>
% if D == 'Poisson':
% if offset:
<% nparams += 1 %>
% endif
% endif

##define the prior...use non-informative prior #fixed effect N(0,10^6), variance~IW(1,0.002)~IG(0.001000,0.001000)
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})))

% if D == 'Normal':
myModel<-MCMCglmm(formula,random=~${L2ID},family="gaussian",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% elif D == 'Binomial':
% if link == 'logit':
denom = sum(mydata$${n}) / length(mydata$${n})
if (denom == 1) {
##binary data=>categorical model
myModel<-MCMCglmm(formula,random=~${L2ID},family="categorical",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
}
if (denom > 1) {
##binomial data
mydata$aY <- mydata$${n} - mydata$${y}
formula <- cbind(${y}, aY) ~ ${Rmmult(x)} - 1
myModel<-MCMCglmm(formula,random=~${L2ID},family="multinomial2",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
}
% elif link == 'probit':
myModel<-MCMCglmm(formula,random=~${L2ID},family="ordinal",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% else:
<% print "This link is not yet implemented, choose another engine" %>
% endif
% elif D == 'Poisson':
##if there is an offset this needs to be added to the model
% if offset:
update(formula, . ~ ., + ${n})
## and constrained the value of the offset parameter
myprior$B$V[${nparams},${nparams}]<-1e-6
myprior$B$mu[${nparams}]<-1
% endif
myModel<-MCMCglmm(formula,random=~${L2ID},family="poisson",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% else:
<% print 'Distribution not implemented yet, choose another engine:' %>
% endif

##summary and outputs from the model
## combined output fixed and variance into one matrix 

summary(myModel)

% if D == 'Binomial':
% if link == 'logit':
c<-((16*sqrt(3))/(15*pi))^2
myModel$Sol<-myModel$Sol/sqrt(1+c*myModel$VCV[,"units"])
myModel$VCV<-myModel$VCV/(1+c*myModel$VCV[,"units"])
% elif link == 'probit':
c<-1
myModel$Sol<-myModel$Sol/sqrt(1+c*myModel$VCV[,"units"])
myModel$VCV<-myModel$VCV/(1+c*myModel$VCV[,"units"])
% endif
% endif

myModelOut<-cbind(myModel$Sol,myModel$VCV)
myModelOut<-mcmc(myModelOut)
summary(myModelOut)

##temporary plots
<% ind=divmod(nparams,3) %>
% for i in range(ind[0]):
<% svgout='DiagPlots'+str(i+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${i}):(3*${i+1})])
dev.off()
% endfor
% if (ind[1]>0):
<% svgout='DiagPlots'+str(ind[0]+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${ind[0]}):${nparams}])
dev.off()
% endif
## export chains
colnames(myModelOut)[which(colnames(myModelOut)=="units")]<-"sigma^2"
write.dta(data.frame(myModelOut),file="chains.dta")
## export DIC
dic <- myModel$DIC
write.dta(data.frame(dic), file = "dic.dta")

% endif

% if Rpackage == 'R_RStan':
<% from EStat.Templating import mmult %>
mydata=as.list(mydata)
mydata$N=length(mydata$${y})
mydata$n2=length(unique(mydata$${L2ID}))
% if D == 'Normal':
scode ="
data {
    int<lower=0> N;
    int<lower=0> n2;
    int<lower=1, upper=n2> ${L2ID}[N];
    real ${y}[N];
    % for i in range(len(x)):
    real ${x[i]}[N];
    % endfor
  
}
parameters {
    % for i in range(len(x)):
    real beta${i};
    % endfor   
  
    real u0[n2];
    real<lower=0> sigmasq_${y};
    real<lower=0> sigmasq_u0;
}
transformed parameters {

    real<lower=0> sigma_${y};
    real<lower=0> sigma_u0;
    

    sigma_${y} <- sqrt(sigmasq_${y});
    sigma_u0 <- sqrt(sigmasq_u0);
    
}
model {

    sigmasq_${y} ~ inv_gamma(0.001, 0.001);
  
    sigmasq_u0 ~ inv_gamma(0.001, 0.001);
    u0 ~ normal(0.0, sigma_u0);
     
    for (n in 1:N){
        ${y}[n] ~ normal(${mmult(x,'beta','n')} + u0[${L2ID}[n]], sigma_${y});
    }

}
"
% elif D == 't':
mydata$df = 8.0
scode ="
data {
    int<lower=0> N;
    int<lower=0> n2;
    int<lower=1, upper=n2> ${L2ID}[N];
    real ${y}[N];
    real<lower=0> df;
    % for i in range(len(x)):
    real ${x[i]}[N];
    % endfor
  
}
parameters {
    % for i in range(len(x)):
    real beta${i};
    % endfor   
  
    real u0[n2];
    real<lower=0> sigmasq_${y};
    real<lower=0> sigmasq_u0;
}
transformed parameters {
    real<lower=0> sigma_${y};
    real<lower=0> sigma_u0;
    
    sigma_${y} <- sqrt(sigmasq_${y});
    sigma_u0 <- sqrt(sigmasq_u0);
    
}
model {
    sigmasq_${y} ~ inv_gamma(0.001, 0.001);
    sigmasq_u0 ~ inv_gamma(0.001, 0.001);
    
    for (t in 1:n2){
        u0[t] ~ student_t(df, 0.0, sigma_u0);
    }
     
    for (n in 1:N){
        ${y}[n] ~ normal(${mmult(x,'beta','n')} + u0[${L2ID}[n]], sigma_${y});
    }

}
"
% elif D == 'Binomial':
scode ="
data {
    int<lower=0> N;
    int<lower=0> n2;
    int<lower=0, upper=n2> ${L2ID}[N];
    int ${y}[N];
    int<lower=0> ${n}[N];
    % for i in range(len(x)):
    real ${x[i]}[N];
    % endfor
      
}
parameters {
    % for i in range(len(x)):
    real beta${i};
    % endfor   
  
    real u0[n2];
    real<lower=0> sigmasq_u0;
}
transformed parameters {

    real<lower=0> sigma_u0;
    sigma_u0 <- sqrt(sigmasq_u0);
    
}
model {
  
    sigmasq_u0 ~ inv_gamma(0.001, 0.001);
    u0 ~ normal(0.0, sigma_u0);
     
    for (n in 1:N){
    % if link == 'logit':
        ${y}[n] ~ binomial(${n}[n], inv_logit(${mmult(x,'beta','n')} + u0[${L2ID}[n]]));
    % elif link == 'probit':
        ${y}[n] ~ binomial(${n}[n], Phi(${mmult(x,'beta','n')} + u0[${L2ID}[n]]));
    % endif
    }

}
"
% elif D == 'Poisson':
scode ="
data {
    int<lower=0> N;
    int<lower=0> n2;
    int<lower=0, upper=n2> ${L2ID}[N];
    int ${y}[N];
    real ${n}[N];
    % for i in range(len(x)):
    real ${x[i]}[N];
    % endfor
      
}
parameters {
    % for i in range(len(x)):
    real beta${i};
    % endfor   
  
    real u0[n2];
    real<lower=0> sigmasq_u0;
}
transformed parameters {

    real<lower=0> sigma_u0;
    sigma_u0 <- sqrt(sigmasq_u0);
    
}
model {
  
    sigmasq_u0 ~ inv_gamma(0.001, 0.001);
    u0 ~ normal(0.0, sigma_u0);
     
    for (n in 1:N){
    % if offset:
        ${y}[n] ~ poisson(exp(${n}[n]+${mmult(x,'beta','n')} + u0[${L2ID}[n]]));
    %else:
        ${y}[n] ~ poisson(exp(${mmult(x,'beta','n')} + u0[${L2ID}[n]]));
    % endif
    }

}
"
% endif
fit <- stan(model_code = scode, data=mydata,iter=${iterations+burnin}, warmup=${burnin}, thin=${thinning}, seed=${seed}, chains=${nchains})
print(fit)
fit_ss <- extract(fit, permuted = TRUE)

myModelOut=NULL
% for i in range(len(x)):
myModelOut=data.frame(cbind(myModelOut,beta${i}=fit_ss$beta${i}))
% endfor
%if storeresid:
for (i in 1:mydata$n2){
    temp=data.frame(fit_ss$u0[,i]); colnames(temp)=paste("u0_",i-1,sep="")
    myModelOut=data.frame(cbind(myModelOut,temp))
}
%endif
% if D == 'Normal':
myModelOut=data.frame(cbind(myModelOut,sigmasq_${y}=fit_ss$sigmasq_${y}))
%endif
myModelOut=data.frame(cbind(myModelOut,sigmasq_u0=fit_ss$sigmasq_u0))
myModelOut=data.frame(cbind(myModelOut,lp__=fit_ss$lp__))
write.dta(myModelOut,file="chains.dta")


svg("stanplot.svg")
plot(fit)
dev.off()
% endif

% if Rpackage == 'mgcv':
mydata$${L2ID}=as.factor(mydata$${L2ID})
formula <- ${y} ~ ${Rmmult(x)} + s(${L2ID}, bs="rw") - 1
% if D == 'Normal':
<% Rfamily='gaussian' %>
myModel = gam(formula, data = mydata, method="REML")
%elif D == 'Binomial':
<% Rfamily='binomial('+str(link)+')' %>
myModel = gam(formula, data = mydata, family=${Rfamily},offset=${n}, method="REML")
%elif D == 'Poisson':
<% Rfamily='poisson' %>
myModel = gam(formula, data = mydata, family=${Rfamily},offset=${n}, method="REML")
% endif
summary(myModel)
#tempory plot
mysvg <- function(filename = "${L2ID}Plot.svg", ...) { svg(filename, ...) } 
options(device="mysvg") 
plot(myModel,pages=1,scheme=1)
dev.off()

mysvg2 <- function(filename = "gamCheck.svg", ...) { svg(filename, ...) } 
options(device="mysvg2") 
gam.check(myModel)
dev.off()

% endif
'''

    statascript = '''
% if D == 'Normal':
<% command = 'xtmixed' %>
% elif D == 'Binomial':
<% command = 'xtmelogit' %>
<% denom=sum(datafile.variables[str(n)]['data'])/len(datafile.variables[str(n)]['data']) %>
% if denom > 1:
<% family += ' ' + n %>
% endif
% if link != 'logit':
/// Stata can only fit logit binomial model !! 
% endif
% elif D == 'Poisson':
<% command = 'xtmepoisson' %>
% endif

${command} ${y} \\
% for p in x:
${p} \\
% endfor
,\\
% if D == 'Poisson':
% if offset:
offset(${n}) \\
% endif
% endif
noconstant\\
||${L2ID}:, \\
${estmethod} variance \\
% if D == 'Normal':
nostderr
% endif

/// Plot of residuals 
predict u0, reffects
predict u0se, reses
egen pickone = tag(${L2ID})
sort u0
generate u0rank = sum(pickone)
serrbar u0 u0se u0rank if pickone==1, scale(1.96) yline(0)
graph export "Cater_resid.png", replace
'''

    spssscript = '''
% if D == 'Normal':
MIXED \\
${y} with \\
% for p in x:
${p} \\
% endfor
/FIXED = \\
% for p in x:
${p} \\
% endfor
| NOINT \\
/RANDOM = INTERCEPT | \\
SUBJECT(${L2ID}) \\
COVTYPE(${randomcov})\\
/METHOD = ${estmethod}\\
/PRINT = SOLUTION G\\
/CRITERIA LCONVERGE(1E-4 ABSOLUTE).
% else :
GENLINMIXED \\
/FIELDS \\
TARGET = ${y} \\
% if D == 'Poisson':
% if offset:
OFFSET=field(${n}) \\
% endif
% endif
/TARGET_OPTIONS \\
% if D == 'Poisson' :
distribution = poisson link=log \\
% elif D == 'Binomial' :
distribution = binomial link=${link} \\
% endif
/FIXED \\
EFFECTS = \\
% for p in x :
${p} \\
% endfor
use_intercept = FALSE \\
/RANDOM \\
subjects = ${L2ID} \\
covariance_type = ${randomcov} \\
use_intercept = TRUE

/BUILD_OPTIONS \\
df_method = satterthwaite \\
/EMMEANS_OPTIONS scale=original padjust=lsd
/SAVE predicted_values confidence_intervals pearson_residuals .
% endif
'''

    genstatscript = '''
<%import numpy %>
UNITS [NVALUES=*]
DELETE[REDEFINE=yes] ${L2ID}
FACTOR [MODIFY=no; NVALUES=${len(datafile['datafile'][str(L2ID)])}; LEVELS=${len(numpy.unique(datafile['datafile'][str(L2ID)]))}; REFERENCE=1] ${L2ID}
READ ${L2ID}; FREPRESENTATION=ordinal
<%
tempstr =  ''
random=datafile['datafile'][str(L2ID)]
count =1
for d in random:
    if count==100:
        tempstr =tempstr +str(d)+' \\n'
        count=1
    else:
        tempstr =tempstr +str(d)+" "
    count+=1
    
tempstr =tempstr +":"
%>
${tempstr}
<%
xplus=''
flagcons=False
x0=[]
for p in range(len(x)):
    if x[p]=='cons':
        flagcons=True
    else:
        x0.append(x[p])

for p in range(len(x0)):        
    if p !=len(x0)-1:
        xplus=xplus+(str(x0[p]))+ '+'
    else:
        xplus=xplus+(str(x0[p]))


%>

% if D=='Normal':
GLMM [PRINT=model,monitor,components,vcovariance,means,backmeans,effects; DISTRIBUTION=normal;\
LINK=identity; DISPERSION=*; FIXED=${xplus}; RANDOM=${L2ID}; CONSTANT=\\
% if flagcons==True:
estimate;\\
% else:
omit;\\
% endif
FACT=9; PSE=differences,estimates; MAXCYCLE=20; FMETHOD=all; MVINCLUDE=*; CADJUST=mean]\
${y}
% elif D=='Binomial':
<% denom=sum(datafile['datafile'][str(n)])/len(datafile['datafile'][str(n)]) %>
GLMM [PRINT=model,monitor,components,vcovariance,means,backmeans,effects; DISTRIBUTION=binomial;\
LINK=${link}; DISPERSION=*; FIXED=${xplus}; RANDOM=${L2ID}; CONSTANT=\\
% if flagcons==True:
estimate;\\
% else:
omit;\\
% endif
FACT=9;PSE=differences,estimates; MAXCYCLE=20; FMETHOD=all; MVINCLUDE=*; CADJUST=mean] ${y};\
NBINOMIAL=\\
% if denom == 1:
1
% else:
${n}
% endif
% elif D=="Poisson":
GLMM [PRINT=model,monitor,components,vcovariance,means,backmeans,effects; DISTRIBUTION=poisson;\
LINK=${link}; DISPERSION=*; FIXED=${xplus}; RANDOM=${L2ID}; CONSTANT=\\
% if flagcons==True:
estimate;\\
% else:
omit;\\
% endif
FACT=9; PSE=differences,estimates; MAXCYCLE=20; FMETHOD=all; MVINCLUDE=*; CADJUST=mean]\
${y}
% endif
ENDJOB
'''

    sasscript = '''
% if D == 'Normal':
proc mixed data=data asycov;
class ${L2ID};
model ${y} = \\
% for p in x:
${p} \\
% endfor
/noint;
random int / subject=${L2ID};
store sasuser.mixed;
run;

proc plm source=sasuser.mixed;
show all;
score data = data out = results;
run;

proc export outfile="results.dta" data=results dbms = dta replace;
% endif

% if D == 'Poisson':
<% params = ['b' + str(b) for b in range(0, len(x))] %>
<% inits = [a + ' = 0' for a in params] %>
<% elements = [a[0] + '*' + a[1] for a in zip(params, list(x))] %>
proc nlmixed data=data;
parms ${" ".join(inits)} sigma2 = 1;
y = \\
% if offset:
${n} + \\
% endif
${' + '.join(list(elements))} + u;
lambda = exp(y);
model ${y} ~ Poisson(lambda);
random u ~ normal(0, sigma2) subject = ${L2ID};
run;
% endif

% if D == 'Binomial':
<% params = ['b' + str(b) for b in range(0, len(x))] %>
<% inits = [a + ' = 0' for a in params] %>
<% elements = [a[0] + '*' + a[1] for a in zip(params, list(x))] %>
proc nlmixed data=data;
parms ${" ".join(inits)} sigma2 = 1;
y = ${' + '.join(list(elements))} + u;
% if link == 'logit':
lambda = exp(y)/(1+exp(y));
% endif
% if link == 'probit':
lambda = probnorm(y);
% endif
% if link == 'cloglog':
lambda = 1 - exp(-exp(y));
% endif
model ${y} ~ Binomial(${n} ,lambda);
random u ~ normal(0, sigma2) subject = ${L2ID};
run;
% endif

% if D == 't':
<% params = ['b' + str(b) for b in range(0, len(x))] %>
<% inits = [a + ' = 0' for a in params] %>
<% elements = [a[0] + '*' + a[1] for a in zip(params, list(x))] %>
proc nlmixed data=data;
parms ${" ".join(inits)} sigma2 = 1 dft = 5;
y = ${' + '.join(list(elements))} + u;
pi = arcos(-1);
z = (${y}-y)/sqrt(sigma2);
lambda=lgamma(.5*(dft+1))-.5*log(sigma2)-.5*log(dft*pi)-lgamma(dft/2) -( (dft+1)/2 )* log( 1+ (1/dft)*z**2);
model ${y} ~ general(lambda);
random u ~ normal(0, sigma2) subject = ${L2ID};
run;
% endif
'''
