# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateNLevXCPrior(Template):
    'Fits a variance components model with one or more higher classifications (nested or crossed), allowing expression of informative priors for all parameters.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal', 'Poisson', 'Binomial', 'Probit', 'Informative priors' ]
    engines = ['eStat', 'WinBUGS', 'OpenBUGS', 'JAGS', 'MLwiN_MCMC', 'R_MCMCglmm']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Poisson'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit', 'cloglog'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)

beta = ParamVector(parents=[x], as_scalar=True)
for i in range(0, beta.ncols):
    context['beta' + str(i) + 'prior'] = List('Prior for beta' + str(i) + ' (mean, sd or n for default): ')
for i in range(0, int(NumLevs)):
    context['tau_u' + str(i + 1) + 'prior'] = List('Prior for tau_u' + str(i + 1) + ' (estimate, sample size or n for default): ')

if D == 'Normal':
    tauprior = List('Prior for tau (estimate, sample size or n for default): ')

storeresid = Boolean('Store residuals?')


if D == 'Normal':
    tau = ParamScalar()
    sigma2 = ParamScalar(modelled = False)
for i in range(0, int(NumLevs)):
    if storeresid:
        context['u' + str(i + 1)] = ParamVector(parents=[context['C' + str(i + 1)]], as_scalar=False)
    else:
        context['u' + str(i + 1)] = ParamVector(parents=[context['C' + str(i + 1)]], as_scalar=False, monitor=False)
    context['tau_u' + str(i + 1)] = ParamScalar()
    context['sigma2_u' + str(i + 1)] = ParamScalar(modelled = False)

deviance = ParamScalar(modelled=False, customstep=True)
'''

    model = '''
<% numlevs = int(NumLevs) %>

model {
        for (i in 1:length(${y})) {
            ${y}[i] ~ \\
            % if D == 'Normal':
dnorm(mu[i], tau)
            mu[i] <- \\
            % endif
            % if D == 'Binomial':
dbin(p[i], ${n}[i])
            ${link}(p[i]) <-
            % endif
            % if D == 'Poisson':
dpois(p[i])
            ${link}(p[i]) <- \\
            % if offset:
            ${n}[i] + \\
            % endif
            % endif 
${mmult(x, 'beta', 'i')} \\
            % for i in range(0, numlevs):
+ u${i + 1}[${context['C' + str(i + 1)]}[i]]
            % endfor
        }

    % for i in range(0, numlevs):
    for (i${i + 1} in 1:length(u${i + 1})) {
        u${i + 1}[i${i + 1}] ~ dnorm(0, tau_u${i + 1})
    }
    % endfor
    
    # Priors
    % for i in range(0, x.ncols()):
    % if context['beta' + str(i) + 'prior'] == 'n':
    beta_${i} ~ dflat()
    % else:
    <% mean = float(context['beta' + str(i) + 'prior'].name[0]) %>
    <% prec = 1.0/pow(float(context['beta' + str(i) + 'prior'].name[1]),2) %>
    beta_${i} ~ dnorm(${mean}, ${prec})
    % endif
    % endfor
    % if D == 'Normal':
    % if context['tauprior'] == 'n':
    tau ~ dgamma(0.001000, 0.001000)
    % else:
    <% est = float(context['tauprior'].name[0]) %>
    <% ss = float(context['tauprior'].name[1]) %>
    tau ~ dgamma(${(ss/2) + 1}, ${(ss/2)*est})
    % endif
    sigma2 <- 1 / tau
    % endif
    % for i in range(0, numlevs):
    % if context['tau_u' + str(i + 1) + 'prior'] == 'n':
    tau_u${i + 1} ~ dgamma(0.001000, 0.001000)
    % else:
    <% est = float(context['tau_u' + str(i + 1) + 'prior'].name[0]) %>
    <% ss = float(context['tau_u' + str(i + 1) + 'prior'].name[1]) %>
    tau_u${i + 1} ~ dgamma(${(ss/2) + 1}, ${(ss/2)*est})
    % endif
    sigma2_u${i + 1} <- 1 / tau_u${i + 1}
    % endfor
}    
'''

    latex = r'''
<% numlevs = int(NumLevs) %>
\begin{aligned} 
%if D == 'Normal':
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
%if D == 'Binomial':
 \mbox{${y}}_{i} & \sim \mbox{Binomial}(\mbox{${n}}_{i}, \pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%endif
%if D == 'Poisson':
 \mbox{${y}}_{i} & \sim \mbox{Poisson}(\pi_{i}) \\ 
\mbox{${link}}(\pi_{i}) & =
%if offset:
\mbox{${n}}_{i} +
%endif
%endif
%if D == 't' :
 \mbox{${y}}_{i} & \sim \mbox{N}(\mu_{i}, \sigma^2) \\ 
\mu_{i} & =
%endif
${mmulttex(x, r'\beta', 'i')} 
% for i in range(0, numlevs):
+ u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]}
%endfor
\\ 
%if D=='Normal':
% for i in range(0, numlevs):
u^{(${i + 2})}_{\mbox{${context['C' + str(i + 1)]}}[i]} & \sim \mbox{N}(0,\sigma^2_{u${i + 1}}) \\ 
%endfor
%endif
%for i in range(0, len(x)):
% if context['beta' + str(i) + 'prior'] == 'n':
\beta_${i} & \propto 1 \\ 
% else:
<% mean = float(context['beta' + str(i) + 'prior'].name[0]) %>
<% sd = pow(float(context['beta' + str(i) + 'prior'].name[1]),2) %>
\beta_${i} & \sim \mbox{N} (${mean}, ${sd}) \\ 
% endif
%endfor
%if D == 'Normal':
% if tauprior == 'n':
\tau & \sim \Gamma (0.001,0.001) \\ 
% else:
<% est = float(tauprior.name[0]) %>
<% ss = float(tauprior.name[1]) %>
\tau & \sim \Gamma (${(ss/2) + 1}, ${(ss/2)*est}) \\ 
% endif
\sigma^2 & = 1 / \tau \\ 
%endif
% for i in range(0, numlevs):
% if context['tau_u' + str(i + 1) + 'prior'] == 'n':
\tau_{u${i + 1}} & \sim \Gamma (0.001,0.001) \\ 
% else:
<% est = float(context['tau_u' + str(i + 1) + 'prior'].name[0]) %>
<% ss = float(context['tau_u' + str(i + 1) + 'prior'].name[1]) %>
\tau_{u${i + 1}} & \sim \Gamma (${(ss/2) + 1}, ${(ss/2)*est}) \\ 
% endif
\sigma_{u${i + 1}}^2 & = 1 / \tau_{u${i + 1}} \\ 
% endfor
\end{aligned} 
'''

    mlnscript = '''
% for i in range(NumLevs):
NOTE CALC "${context['C' + str(i + 1)]}" = "${context['C' + str(i + 1)]}" + 1
% endfor
RESP "${y}"
% for i in range(NumLevs):
IDEN ${i + 2} "${context['C' + str(i + 1)]}"
% endfor
IDEN 1 "_id"
% if D == 'Binomial':
RDISt 1 0
DOFFs 1 "${n}"
% if link == 'logit':
LFUN 0
% elif link == 'probit':
LFUN 1
% elif link == 'cloglog':
LFUN 2
% endif
ADDT "_levres"
% for i in range(1, NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% elif D == 'Normal':
ADDT "_levres"
% for i in range(NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% elif D == 'Poisson':
RDISt 1 1
LFUN 3
% if offset:
DOFFs 1 ${n}
% endif
ADDT "_levres"
% for i in range(1, NumLevs + 1):
SETV ${i + 1} "_levres"
% endfor
FPAR 0 "_levres"
% for i in range(len(x)):
ADDT "${x[i]}"
% endfor
% endif

<%
T=''
for i in range(0, beta.ncols):
    if context['beta' + str(i) + 'prior'] != 'n':
        mean = float(context['beta' + str(i) + 'prior'].name[0])
        sd = float(context['beta' + str(i) + 'prior'].name[1])
        T=T+str(1)+' '+str(mean)+' '+str(sd)+' '
    else:
        T=T+str(0)+' '

if D=='Binomial':
    for i in range(0, NumLevs):
        if context['tau_u' + str(i + 1) + 'prior'] != 'n':
            est = float(context['tau_u' + str(i + 1) + 'prior'].name[0])
            ss = float(context['tau_u' + str(i + 1) + 'prior'].name[1])
            T=T+str(1)+' '+str(est)+' '+str(ss)+' '
        else:
            T=T+str(0)+' '
    T=T+str(0)+' '
elif D=='Normal':
    for i in range(0, NumLevs):
        if context['tau_u' + str(i + 1) + 'prior'] != 'n':
            est = float(context['tau_u' + str(i + 1) + 'prior'].name[0])
            ss = float(context['tau_u' + str(i + 1) + 'prior'].name[1])
            T=T+str(1)+' '+str(est)+' '+str(ss)+' '
        else:
            T=T+str(0)+' '
    if context['tauprior'] != 'n':
        est = float(context['tauprior'].name[0])
        ss = float(context['tauprior'].name[1])
        T=T+str(1)+' '+str(est)+' '+str(ss)+' '
    else:
        T=T+str(0)+' '
elif D=='Poisson':
    for i in range(0, NumLevs):
        if context['tau_u' + str(i + 1) + 'prior'] != 'n':
            est = float(context['tau_u' + str(i + 1) + 'prior'].name[0])
            ss = float(context['tau_u' + str(i + 1) + 'prior'].name[1])
            T=T+str(1)+' '+str(est)+' '+str(ss)+' '
        else:
            T=T+str(0)+' '
    T=T+str(0)+' '
%>
JOIN ${T} c1092 
PRIOR  c1092 
XCLA 1
'''

    rscript = '''
<%
import re
from EStat.Templating import Rmmult
%>

% if Rpackage == 'MCMCglmm':
formula <- ${y} ~ ${Rmmult(x)} - 1

<% nparams=len(x) %>
% if D == 'Poisson':
% if offset:
<% nparams += 1 %>
% endif
% endif


##define the prior...use non-informative prior #fixed effect N(0,10^6), variance~IW(1,0.002)~IG(0.001000,0.001000)
myprior<-list(B=list(V=diag(${nparams})*10^6,mu=rep(0,${nparams})))
%for i in range(0, beta.ncols):
%if context['beta' + str(i) + 'prior'] != 'n':
myprior$B$mu[${i+1}] = ${float(context['beta' + str(i) + 'prior'].name[0])}
myprior$B$V[${i+1},${i+1}] = ${float(context['beta' + str(i) + 'prior'].name[1])**2.0}
%endif
%endfor

%if D=='Binomial' or D=='Poisson':
%for i in range(0, NumLevs):
%if context['tau_u' + str(i + 1) + 'prior'] != 'n':
myprior$G$G${i+1}$V = ${float(context['tau_u' + str(i + 1) + 'prior'].name[0])}
myprior$G$G${i+1}$nu= ${float(context['tau_u' + str(i + 1) + 'prior'].name[1])}
%else:
myprior$G$G${i+1}$V = 1
myprior$G$G${i+1}$nu= 0
%endif
%endfor
%elif D=='Normal':
%for i in range(0, NumLevs):
%if context['tau_u' + str(i + 1) + 'prior'] != 'n':
myprior$G$G${i+1}$V = ${float(context['tau_u' + str(i + 1) + 'prior'].name[0])}
myprior$G$G${i+1}$nu= ${float(context['tau_u' + str(i + 1) + 'prior'].name[1])}
%else:
myprior$G$G${i+1}$V = 1
myprior$G$G${i+1}$nu= 0
%endif
%endfor
%if context['tauprior'] != 'n':
myprior$R$V = ${float(context['tauprior'].name[0])}
myprior$R$nu= ${float(context['tauprior'].name[1])}
%else:
myprior$R$V = 1
myprior$R$nu= 0
%endif
%endif




% if D == 'Normal':
myModel<-MCMCglmm(formula,random=~${context['C1']}\\
% for i in range(1,int(NumLevs)):
+ ${context['C'+str(i + 1)]}\\
% endfor
,family="gaussian",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% elif D == 'Binomial':
% if link == 'logit':
denom = sum(mydata$${n}) / length(mydata$${n})
##evaluate to the denominator to see if it is binary or binomial model
if (denom == 1) {
##binary data=>categorical model
myModel<-MCMCglmm(formula,random=~${context['C1']}\\
% for i in range(1,int(NumLevs)):
+ ${context['C' + str(i + 1)]}\\
% endfor
,family="categorical",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
} else {
##binomial data
myData$aY<-myData$${n} - myData$${y}
formula <- cbind(${y}, aY) ~ ${Rmmult(x)} - 1
myModel<-MCMCglmm(formula,random=~${context['C1']}\\
% for i in range(1,int(NumLevs)):
+ ${context['C' + str(i + 1)]}\\
% endfor
,family="multinomial2",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
}
% elif link == 'probit':
myModel<-MCMCglmm(formula,random=~${context['C1']}\\
% for i in range(1,int(NumLevs)):
+ ${context['C' + str(i + 1)]}\\
% endfor
,family="ordinal",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% else:
<% print "This link is not yet implemented, choose another engine" %>
% endif
% elif D == 'Poisson':
##if there is an offset this needs to be added to the model
% if offset:
update(formula, . ~ ., + ${n})
## and constrained the value of the offset parameter
myprior$B$V[${nparams},${nparams}]<-1e-6
myprior$B$mu[${nparams}]<-1
% endif
myModel<-MCMCglmm(formula,random=~${context['C1']}\\
% for i in range(1,int(NumLevs)):
+ ${context['C' + str(i + 1)]}\\
% endfor
,family="poisson",prior=myprior,burnin=${burnin},nitt=${iterations + burnin},thin=${thinning},data=mydata)
% else:
<% print 'Distribution not implemented yet, choose another engine:' %>
% endif

## summary and outputs from the model
## combined output fixed and variance into one matrix 

summary(myModel)

% if D == 'Binomial':
% if link == 'logit':
c<-((16*sqrt(3))/(15*pi))^2
myModel$Sol<-myModel$Sol/sqrt(1+c*myModel$VCV[,"units"])
myModel$VCV<-myModel$VCV/(1+c*myModel$VCV[,"units"])
% elif link == 'probit':
c<-1
myModel$Sol<-myModel$Sol/sqrt(1+c*myModel$VCV[,"units"])
myModel$VCV<-myModel$VCV/(1+c*myModel$VCV[,"units"])
% endif
% endif

myModelOut<-cbind(myModel$Sol,myModel$VCV)
myModelOut<-mcmc(myModelOut)
summary(myModelOut)

##temporary plots
<% ind=divmod(nparams,3) %>

% for i in range(ind[0]):
<% svgout='DiagPlots'+str(i+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${i}):(3*${i+1})])
dev.off()
% endfor

% if (ind[1]>0):
<% svgout='DiagPlots'+str(ind[0]+1)+'.svg' %>
svg("${svgout}")
plot(myModelOut[,(1+3*${ind[0]}):${nparams}])
dev.off()
% endif

## export chains
colnames(myModelOut)[which(colnames(myModelOut)=="units")]<-"sigma^2"
write.dta(data.frame(myModelOut),file="chains.dta")
## export DIC
dic <- myModel$DIC
write.dta(data.frame(dic), file = "dic.dta")

% endif
'''
